/******************************************************************************
 * minimos.c
 * Federico Bareilles
 * 08/02/2003
 *
 * Minimos cuadrados no lineales.
 *
 *****************************************************************************/

#define _POSIX_SOURCE 1         /* POSIX compliant source  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "matrix.h"
#include "minimos.h"

//static double DT = 20.0;

double mi_modelo(double x, double *xx)
{
  double c = *(xx+0);
  double t = *(xx+1);
  double l = *(xx+2);
  double a = *(xx+3);

  return c * (l + a * (exp(-1.0*t * x)-1.0));
}

double mi_deriv_c(double x, double *xx)
{
  //double c = *(xx+0);
  double t = *(xx+1);
  double l = *(xx+2);
  double a = *(xx+3);

  return (l + a * (exp(-1.0*t * x)-1.0));
}

double mi_deriv_t(double x, double *xx)
{
  double c = *(xx+0);
  double t = *(xx+1);
  //double l = *(xx+2);
  double a = *(xx+3);

  return -1.0 * c * a * x * exp(-1.0*t * x);
}

double mi_deriv_l(double x, double *xx)
{
  double c = *(xx+0);
  //double t = *(xx+1);
  //double l = *(xx+2);
  //double a = *(xx+3);

  return c /** exp(-t * x)*/;
}

double mi_deriv_a(double x, double *xx)
{
  double c = *(xx+0);
  double t = *(xx+1);
  //double l = *(xx+2);
  //double a = *(xx+3);

  return c * (exp(-1.0*t * x)-1.0);
}




int main(void) /* only for test */
{
  int i;
  double x[4];
  
  struct model *m;

  double xdata[] = {1.084, 1.399, 1.752, 1.930, 2.158, 2.459, 2.869, 3.461};
  //double xdata[] = {1.084, 1.399, 1.752, 1.930, 2.158, 2.459, 2.869, 3.461};
  double ydata[] = {101.189, 94.651, 85.243, 81.244, 
  		   73.910, 63.623, 48.566, 30.345};
  //double ydata[] = {100.582, 93.4205, 83.5381, 79.2941, 71.8405, 61.3962, 46.4777, 28.6457};
  double sigma[] = {1.844, 1.939, 1.696, 1.615, 1.843, 1.679, 1.754, 1.780};
  //double sigma[] = {1.84378, 1.93889, 1.69562, 1.61538, 1.84283, 1.67904, 1.75382, 1.78044};

  m = make_model(mi_modelo);
  model_add_deriv(m, mi_deriv_c);
  model_add_deriv(m, mi_deriv_t);
  model_add_deriv(m, mi_deriv_l);
  //model_add_deriv(m, mi_deriv_a);

  m->xdata = xdata;
  m->ydata = ydata;
  m->sigma = sigma;
  m->size = 8;

  x[0] = 18.9096;    //c
  x[1] = 0.0035092;  //t
  x[2] = 7.29345;     //l
  x[3] = 463.895;   //a


  fit_model(m,x);
  /*
  printf("modelo  %f\n",m->modelo(1.084,x));
  printf("deriv_c %f\n",m->deriv->func(1.084,x));
  printf("deriv_a %f\n",m->deriv->next->func(1.084,x));
  printf("deriv_t %f\n",m->deriv->next->next->func(1.084,x));
  */

  //print_matrix(m->VCV);
  
  for(i=0;i<m->parametros;i++)
    printf("# %d %f %1.3f\n",i,x[i],sqrt(m->VCV->M[i][i]));
  

  for(i=0;i<m->size;i++){
    printf("%5.3f %8.3f %8.3f\n", 
	   xdata[i], ydata[i], m->modelo(xdata[i],x));
  }



  return 0;
}
